@@ -1,6 +1,6 @@
 /*
   spamdyke -- a filter for stopping spam at connection time.
-  Copyright (C) 2014 Sam Clippinger (samc (at) silence (dot) org)
+  Copyright (C) 2015 Sam Clippinger (samc (at) silence (dot) org)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
@@ -212,12 +212,15 @@ int search_file(struct qrv_settings *current_settings, char *search_filename, ch
   char lower_end_wildcard;
   char lower_target_string[MAX_BUF + 1];
   int strlen_lower_target_string;
+  struct stat tmp_stat;
 
   return_value = 0;
 
   if ((target_string != NULL) &&
       (strlen_target_string > 0))
     {
+    QRV_LOG_EXCESSIVE(current_settings, LOG_EXCESSIVE_SEARCH_FILE, search_filename, strlen_target_string, target_string);
+
     if ((tmp_file = fopen(search_filename, "r")) != NULL)
       {
       line_num = 0;
@@ -250,6 +253,9 @@ int search_file(struct qrv_settings *current_settings, char *search_filename, ch
 
       fclose(tmp_file);
       }
+    else if ((stat(search_filename, &tmp_stat) == -1) &&
+             (errno == ENOENT))
+      return_value = 0;
     else
       {
       QRV_LOG_ERROR(current_settings, LOG_ERROR_OPEN_SEARCH "%s: %s", search_filename, strerror(errno));
@@ -517,20 +523,20 @@ char *reassemble_address(char *target_username, int strlen_username, char *targe
           (target_domain[0] != '\0'))
         {
         if (strlen_username == -1)
-          snprintf(return_address, max_return_address, "%s@%s%n", target_username, target_domain, &tmp_strlen);
+          tmp_strlen = SNPRINTF(return_address, max_return_address, "%s@%s", target_username, target_domain);
         else
-          snprintf(return_address, max_return_address, "%.*s@%s%n", strlen_username, target_username, target_domain, &tmp_strlen);
+          tmp_strlen = SNPRINTF(return_address, max_return_address, "%.*s@%s", strlen_username, target_username, target_domain);
         }
       else
         if (strlen_username == -1)
-          snprintf(return_address, max_return_address, "%s%n", target_username, &tmp_strlen);
+          tmp_strlen = SNPRINTF(return_address, max_return_address, "%s", target_username);
         else
-          snprintf(return_address, max_return_address, "%.*s%n", strlen_username, target_username, &tmp_strlen);
+          tmp_strlen = SNPRINTF(return_address, max_return_address, "%.*s", strlen_username, target_username);
     else if ((target_domain != NULL) &&
              (target_domain[0] != '\0'))
-      snprintf(return_address, max_return_address, "@%s%n", target_domain, &tmp_strlen);
+      tmp_strlen = SNPRINTF(return_address, max_return_address, "@%s", target_domain);
     else if (missing_data != NULL)
-      snprintf(return_address, max_return_address, "%s%n", missing_data, &tmp_strlen);
+      tmp_strlen = SNPRINTF(return_address, max_return_address, "%s", missing_data);
     else
       return_address[0] = '\0';
     }
@@ -709,12 +715,12 @@ int find_path(struct qrv_settings *current_settings, char *filename, char *retur
         {
         if ((tmp_end = strchr(tmp_start, ENVIRONMENT_SEPARATOR)) != NULL)
           {
-          snprintf(new_filename, MAX_BUF, "%.*s" DIR_DELIMITER_STR "%s%n", (int)(tmp_end - tmp_start), tmp_start, filename, &strlen_filename);
+          strlen_filename = SNPRINTF(new_filename, MAX_BUF, "%.*s" DIR_DELIMITER_STR "%s", (int)(tmp_end - tmp_start), tmp_start, filename);
           tmp_start = tmp_end + 1;
           }
         else
           {
-          snprintf(new_filename, MAX_BUF, "%s" DIR_DELIMITER_STR "%s%n", tmp_start, filename, &strlen_filename);
+          strlen_filename = SNPRINTF(new_filename, MAX_BUF, "%s" DIR_DELIMITER_STR "%s", tmp_start, filename);
           tmp_start = NULL;
           }
 
