@@ -1,6 +1,6 @@
 /*
   spamdyke -- a filter for stopping spam at connection time.
-  Copyright (C) 2014 Sam Clippinger (samc (at) silence (dot) org)
+  Copyright (C) 2015 Sam Clippinger (samc (at) silence (dot) org)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
@@ -19,6 +19,14 @@
 #ifndef VALIDATE_QRV_H
 #define VALIDATE_QRV_H
 
-int validate(struct qrv_settings *current_settings, char *target_recipient_username, int strlen_recipient_username, char *target_recipient_domain, int forwarded);
+struct validate_history
+  {
+  char *username;
+  int strlen_username;
+  char *domain;
+  struct validate_history *previous;
+  };
+
+int validate(struct qrv_settings *current_settings, char *target_recipient_username, int strlen_recipient_username, char *target_recipient_domain, struct validate_history *last);
 
 #endif /* VALIDATE_QRV_H */
