@@ -1,6 +1,6 @@
 /*
   spamdyke -- a filter for stopping spam at connection time.
-  Copyright (C) 2014 Sam Clippinger (samc (at) silence (dot) org)
+  Copyright (C) 2015 Sam Clippinger (samc (at) silence (dot) org)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
@@ -307,7 +307,7 @@ int search_morercpthosts_cdb(struct filter_settings *current_settings, char *sea
       {
       if (fread(indexes, sizeof(struct cdb_index), CDB_INDEX_COUNT, tmp_file) == CDB_INDEX_COUNT)
         {
-        snprintf(tmp_key, MAX_BUF, "%.*s%n", strlen_target_domain, target_domain, &tmp_strlen);
+        tmp_strlen = SNPRINTF(tmp_key, MAX_BUF, "%.*s", strlen_target_domain, target_domain);
         return_value = find_cdb_record(current_settings, NULL, 0, search_filename, tmp_key, tmp_strlen, tmp_file, indexes);
         }
       else
@@ -373,7 +373,7 @@ int validate_cdb(struct filter_settings *current_settings, char *target_cdb, cha
             if ((tmp_index[i].num_slots > 0) &&
                 ((tmp_index[i].offset + (tmp_index[i].num_slots * sizeof(struct cdb_hash_slot))) > tmp_stat.st_size))
               {
-              SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_MAIN_OFFSET, source_option, i, tmp_index[i].offset, tmp_stat.st_size);
+              SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_MAIN_OFFSET, source_option, i, tmp_index[i].offset, tmp_stat.st_size);
               return_value = 0;
               }
 
@@ -414,72 +414,72 @@ int validate_cdb(struct filter_settings *current_settings, char *target_cdb, cha
 
                                     if (tmp_hash != tmp_hash_slot.hash_value)
                                       {
-                                      SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_HASH_MISMATCH, source_option, tmp_hash, tmp_hash_slot.hash_value, tmp_record.key_length, tmp_key, tmp_record.key_length, target_cdb);
+                                      SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_HASH_MISMATCH, source_option, tmp_hash, tmp_hash_slot.hash_value, tmp_record.key_length, tmp_key, tmp_record.key_length, target_cdb);
                                       return_value = 0;
                                       }
 
                                     if ((tmp_hash % CDB_INDEX_COUNT) != i)
                                       {
-                                      SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_MAIN_MISMATCH, source_option, i, tmp_hash % CDB_INDEX_COUNT, tmp_hash, tmp_record.key_length, tmp_key, target_cdb);
+                                      SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_MAIN_MISMATCH, source_option, i, tmp_hash % CDB_INDEX_COUNT, tmp_hash, tmp_record.key_length, tmp_key, target_cdb);
                                       return_value = 0;
                                       }
                                     }
                                   else
                                     {
-                                    SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_READ_ERROR, source_option, tmp_record.key_length, target_cdb);
+                                    SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_READ_ERROR, source_option, tmp_record.key_length, target_cdb);
                                     return_value = 0;
                                     }
                                 else
-                                  SPAMDYKE_LOG_INFO(current_settings, CONFIG_TEST_WARNING_CDB_KEY_OVERLENGTH, source_option, i, j, tmp_record.key_length, MAX_BUF);
+                                  SPAMDYKE_LOG_CONFIG_TEST_INFO(current_settings, CONFIG_TEST_WARNING_CDB_KEY_OVERLENGTH, source_option, i, j, tmp_record.key_length, MAX_BUF);
                               else
                                 {
-                                SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_RECORD_LENGTH, source_option, i, j, (tmp_hash_slot.offset + sizeof(struct cdb_record_header) + tmp_record.key_length + tmp_record.data_length), tmp_stat.st_size);
+                                SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_RECORD_LENGTH, source_option, i, j, (tmp_hash_slot.offset + sizeof(struct cdb_record_header) + tmp_record.key_length + tmp_record.data_length), tmp_stat.st_size);
                                 return_value = 0;
                                 }
                               }
                             else
                               {
-                              SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_READ_ERROR, source_option, sizeof(struct cdb_record_header), target_cdb);
+                              SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_READ_ERROR, source_option, sizeof(struct cdb_record_header), target_cdb);
                               return_value = 0;
                               }
                           else
                             {
-                            SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_SEEK_ERROR, source_option, tmp_hash_slot.offset, target_cdb, strerror(errno));
+                            SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_SEEK_ERROR, source_option, tmp_hash_slot.offset, target_cdb, strerror(errno));
                             return_value = 0;
                             }
                         else
                           {
-                          SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_SECONDARY_OFFSET, source_option, i, j, tmp_hash_slot.offset, tmp_stat.st_size);
+                          SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_SECONDARY_OFFSET, source_option, i, j, tmp_hash_slot.offset, tmp_stat.st_size);
                           return_value = 0;
                           }
                         }
                       }
                     else
                       {
-                      SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_READ, source_option, target_cdb, strerror(errno));
+                      SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_READ, source_option, target_cdb, strerror(errno));
                       return_value = 0;
                       }
                   else
                     {
-                    SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_SEEK_ERROR, source_option, tmp_index[i].offset + (j * sizeof(struct cdb_hash_slot)), target_cdb, strerror(errno));
+                    SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_SEEK_ERROR, source_option, tmp_index[i].offset + (j * sizeof(struct cdb_hash_slot)), target_cdb, strerror(errno));
                     return_value = 0;
                     }
                 }
             }
           else
-            SPAMDYKE_LOG_INFO(current_settings, CONFIG_TEST_WARNING_CDB_EMPTY, source_option, target_cdb);
+            SPAMDYKE_LOG_CONFIG_TEST_INFO(current_settings, CONFIG_TEST_WARNING_CDB_EMPTY, source_option, target_cdb);
           }
         else
-          SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_READ_ERROR, source_option, sizeof(struct cdb_index) * CDB_INDEX_COUNT, target_cdb);
+          SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_READ_ERROR, source_option, sizeof(struct cdb_index) * CDB_INDEX_COUNT, target_cdb);
 
         fclose(tmp_cdb);
         }
       else
-        SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_READ, source_option, target_cdb, strerror(errno));
+        SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_READ, source_option, target_cdb, strerror(errno));
     else
-      SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_TOO_SMALL, source_option, target_cdb, tmp_stat.st_size, sizeof(struct cdb_index) * CDB_INDEX_COUNT);
+      SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_TOO_SMALL, source_option, target_cdb, tmp_stat.st_size, sizeof(struct cdb_index) * CDB_INDEX_COUNT);
   else
-    SPAMDYKE_LOG_CONFIG_TEST(current_settings, CONFIG_TEST_ERROR_CDB_MISSING, source_option, target_cdb, strerror(errno));
+    SPAMDYKE_LOG_CONFIG_TEST_ERROR(current_settings, CONFIG_TEST_ERROR_CDB_MISSING, source_option, target_cdb, strerror(errno));
 
   return(return_value);
   }
