@@ -1,6 +1,6 @@
 /*
   spamdyke -- a filter for stopping spam at connection time.
-  Copyright (C) 2014 Sam Clippinger (samc (at) silence (dot) org)
+  Copyright (C) 2015 Sam Clippinger (samc (at) silence (dot) org)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
@@ -152,16 +152,25 @@ char *nihdns_type_name(int target_type)
   }
 
 /*
+ * The return value must only be used to skip bytes in the DNS packet, not used
+ * as the length of the string in return_buf.  That string will be null-
+ * terminated, but the length will be returned in strlen_return_buf.  Because
+ * of DNS packet compression, the two values may be very different.
+ *
  * Return value:
  *   ERROR: -1
  *   SUCCESS: number of bytes to skip
  */
-int nihdns_expand(char *answer_start, char *answer_end, char *answer_ptr, char *return_buf, int length_return_buf)
+int nihdns_expand(struct filter_settings *current_settings, char *answer_start, char *answer_end, char *answer_ptr, char *return_buf, int length_return_buf, int *strlen_return_buf)
   {
   int return_value;
   char *tmp_ptr;
+  int return_len;
+  int found_pointer;
 
   return_value = 0;
+  return_len = 0;
+  found_pointer = 0;
 
   if ((answer_start != NULL) &&
       (answer_end != NULL) &&
@@ -176,8 +185,14 @@ int nihdns_expand(char *answer_start, char *answer_end, char *answer_ptr, char *
            (return_value < length_return_buf))
       if ((tmp_ptr[0] & 0xC0) == 0xC0)
         {
-        if (((tmp_ptr = answer_start + ((tmp_ptr[0] & 0x3F) << 6) + tmp_ptr[1]) > answer_end) ||
-            (tmp_ptr < answer_start))
+        SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_EXPAND_POINTER, tmp_ptr - answer_ptr, NIHDNS_GETINT16(tmp_ptr) & 0x3FFF);
+        if (((tmp_ptr = answer_start + (NIHDNS_GETINT16(tmp_ptr) & 0x3FFF)) <= answer_end) &&
+            (tmp_ptr > answer_start))
+          {
+          return_value += 2;
+          found_pointer = 1;
+          }
+        else
           {
           return_value = -1;
           break;
@@ -185,15 +200,20 @@ int nihdns_expand(char *answer_start, char *answer_end, char *answer_ptr, char *
         }
       else if (tmp_ptr[0] > 0)
         {
-        if (return_value > 0)
+        if (return_len > 0)
           {
-          return_buf[return_value] = '.';
-          return_value++;
+          return_buf[return_len] = '.';
+          return_len++;
           }
 
-        memcpy(return_buf + return_value, tmp_ptr + 1, MINVAL(tmp_ptr[0], length_return_buf - return_value));
-        return_value += tmp_ptr[0];
-        return_buf[return_value] = '\0';
+        memcpy(return_buf + return_len, tmp_ptr + 1, MINVAL(tmp_ptr[0], length_return_buf - return_len));
+        return_len += MINVAL(tmp_ptr[0], length_return_buf - return_len);
+        return_buf[return_len] = '\0';
+        SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_EXPAND_STRING, tmp_ptr - answer_ptr, tmp_ptr[0], return_len);
+
+        if (!found_pointer)
+          return_value += tmp_ptr[0] + 1;
+
         if (((tmp_ptr += tmp_ptr[0] + 1) > answer_end) ||
             (tmp_ptr < answer_start))
           {
@@ -202,9 +222,20 @@ int nihdns_expand(char *answer_start, char *answer_end, char *answer_ptr, char *
           }
         }
       else
+        {
+        if (!found_pointer)
+          return_value++;
+
         break;
+        }
     }
 
+  SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_EXPAND_RESULT, return_len, return_value, return_buf);
+
+  if ((return_value != -1) &&
+      (strlen_return_buf != NULL))
+    *strlen_return_buf = return_len;
+
   return(return_value);
   }
 
@@ -428,7 +459,7 @@ int nihdns_initialize(struct filter_settings *current_settings, int close_socket
             free_string_array(&current_settings->current_options->nihdns_primary_server_list, current_settings->base_options.nihdns_primary_server_list);
             free_string_array(&current_settings->current_options->nihdns_secondary_server_list, current_settings->base_options.nihdns_secondary_server_list);
 
-            snprintf(tmp_ip, MAX_BUF, "%s:%d%n", LOCALHOST_IP, default_port, &strlen_tmp_ip);
+            strlen_tmp_ip = SNPRINTF(tmp_ip, MAX_BUF, "%s:%d", LOCALHOST_IP, default_port);
             append_string(current_settings, &current_settings->current_options->nihdns_primary_server_list, tmp_ip, strlen_tmp_ip);
 
             current_settings->current_options->nihdns_primary_server_data[0].sin_family = AF_INET;
@@ -1312,6 +1343,7 @@ int nihdns_rbl(struct filter_settings *current_settings, char **target_name_arra
   int num_queries;
   int exit_loop;
   int target_name_index;
+  int strlen_name;
   struct previous_action current_lookup;
   struct previous_action *tmp_lookup;
   char **cname_array;
@@ -1331,8 +1363,10 @@ int nihdns_rbl(struct filter_settings *current_settings, char **target_name_arra
       (target_rbl_array != NULL) &&
       (answer_length = nihdns_query(current_settings, target_name_array, current_settings->current_options->nihdns_query_type_rbl, 0, answer, MAX_DNS_PACKET_BYTES_UDP, &answer_ptr, &target_name_index)) > 0)
     {
-    // This seems silly, but it defeats a strict-aliasing warning from gcc when
-    // a more conventional cast is used: (struct nihdns_header *)&answer
+    /*
+     * This seems silly, but it defeats a strict-aliasing warning from gcc when
+     * a more conventional cast is used: (struct nihdns_header *)&answer
+     */
     recast.char_ptr = answer;
     num_answers = ntohs((unsigned short)recast.header_ptr->ancount);
     num_cnames = 0;
@@ -1432,8 +1466,8 @@ int nihdns_rbl(struct filter_settings *current_settings, char **target_name_arra
 
         j = 0;
         for (i = 0; (i < num_cnames) && !return_value; i++)
-          if ((size = nihdns_expand(answer, answer + answer_length, cname_ptr[i], (char *)host, MAX_HOSTNAME)) >= 0)
-            if ((cname_array[j] = (char *)malloc(sizeof(char) * (size + 1))) != NULL)
+          if ((size = nihdns_expand(current_settings, answer, answer + answer_length, cname_ptr[i], (char *)host, MAX_HOSTNAME, &strlen_name)) >= 0)
+            if ((cname_array[j] = (char *)malloc(sizeof(char) * (strlen_name + 1))) != NULL)
               {
               num_queries = 0;
               tmp_lookup = &current_lookup;
@@ -1450,8 +1484,8 @@ int nihdns_rbl(struct filter_settings *current_settings, char **target_name_arra
                   (num_queries < MAX_DNS_QUERIES))
                 {
                 SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_CNAME, target_name_array[target_name_index], host);
-                memcpy(cname_array[j], host, sizeof(char) * size);
-                cname_array[j][size] = '\0';
+                memcpy(cname_array[j], host, sizeof(char) * strlen_name);
+                cname_array[j][strlen_name] = '\0';
                 cname_array[j + 1] = NULL;
                 j++;
                 }
@@ -1460,7 +1494,7 @@ int nihdns_rbl(struct filter_settings *current_settings, char **target_name_arra
               }
             else
               {
-              SPAMDYKE_LOG_ERROR(current_settings, LOG_ERROR_MALLOC, sizeof(char) * (size + 1));
+              SPAMDYKE_LOG_ERROR(current_settings, LOG_ERROR_MALLOC, sizeof(char) * (strlen_name + 1));
               return_value = -1;
               break;
               }
@@ -1509,6 +1543,7 @@ int nihdns_ptr_lookup(struct filter_settings *current_settings, char *target_nam
   int num_answers;
   int num_queries;
   int exit_loop;
+  int strlen_host;
   struct previous_action current_lookup;
   struct previous_action *tmp_lookup;
   char *target_name_array[2];
@@ -1529,8 +1564,10 @@ int nihdns_ptr_lookup(struct filter_settings *current_settings, char *target_nam
   if ((target_name != NULL) &&
       ((answer_length = nihdns_query(current_settings, target_name_array, current_settings->current_options->nihdns_query_type_ptr, 0, answer, MAX_DNS_PACKET_BYTES_UDP, &answer_ptr, NULL)) > 0))
     {
-    // This seems silly, but it defeats a strict-aliasing warning from gcc when
-    // a more conventional cast is used: (struct nihdns_header *)&answer
+    /*
+     * This seems silly, but it defeats a strict-aliasing warning from gcc when
+     * a more conventional cast is used: (struct nihdns_header *)&answer
+     */
     recast.char_ptr = answer;
     num_answers = ntohs((unsigned short)recast.header_ptr->ancount);
     num_cnames = 0;
@@ -1539,9 +1576,9 @@ int nihdns_ptr_lookup(struct filter_settings *current_settings, char *target_nam
     for (i = 0; (i < num_answers) && !return_value && !exit_loop; i++)
       if ((size = nihdns_skip(answer_ptr, answer + answer_length)) >= 0)
         {
-        answer_ptr += size;
+        answer_ptr += size; /* qdata */
         type = NIHDNS_GETINT16(answer_ptr);
-        answer_ptr += sizeof(uint16_t);
+        answer_ptr += sizeof(uint16_t); /* type */
         answer_ptr += sizeof(uint16_t); /* class */
         answer_ptr += sizeof(uint32_t); /* ttl */
         answer_ptr += sizeof(uint16_t); /* size */
@@ -1549,21 +1586,22 @@ int nihdns_ptr_lookup(struct filter_settings *current_settings, char *target_nam
         switch (type)
           {
           case NIHDNS_TYPE_PTR:
-            if ((size = nihdns_expand(answer, answer + answer_length, answer_ptr, (char *)host, MAX_HOSTNAME)) > 0)
-              {
-              SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_PTR, target_name, size, size, host);
+            if ((size = nihdns_expand(current_settings, answer, answer + answer_length, answer_ptr, (char *)host, MAX_HOSTNAME, &strlen_host)) > 0)
+              if (strlen_host > 0)
+                {
+                SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_PTR, target_name, strlen_host, strlen_host, host);
 
-              current_settings->strlen_server_name = size;
-              for (i = 0; i < size; i++)
-                current_settings->server_name[i] = tolower((int)host[i]);
+                current_settings->strlen_server_name = strlen_host;
+                for (i = 0; i < strlen_host; i++)
+                  current_settings->server_name[i] = tolower((int)host[i]);
 
-              current_settings->server_name[current_settings->strlen_server_name] = '\0';
+                current_settings->server_name[current_settings->strlen_server_name] = '\0';
 
-              return_value = 1;
-              answer_ptr += size;
-              }
-            else if (size == 0)
-              SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_EMPTY_DATA, NULL);
+                return_value = 1;
+                answer_ptr += size;
+                }
+              else
+                SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_EMPTY_DATA, NULL);
             else
               {
               SPAMDYKE_LOG_VERBOSE(current_settings, LOG_VERBOSE_DNS_COMPRESSION, target_name);
@@ -1597,7 +1635,7 @@ int nihdns_ptr_lookup(struct filter_settings *current_settings, char *target_nam
         }
 
     for (i = 0; (i < num_cnames) && !return_value; i++)
-      if ((size = nihdns_expand(answer, answer + answer_length, cname_ptr[i], (char *)host, MAX_HOSTNAME)) >= 0)
+      if ((size = nihdns_expand(current_settings, answer, answer + answer_length, cname_ptr[i], (char *)host, MAX_HOSTNAME, NULL)) >= 0)
         {
         current_lookup.data = target_name;
         current_lookup.count++;
@@ -1692,8 +1730,10 @@ int nihdns_a_inner(struct filter_settings *current_settings, char *target_name,
   if ((target_name != NULL) &&
       ((answer_length = nihdns_query(current_settings, target_name_array, target_query_type, 0, answer, MAX_DNS_PACKET_BYTES_UDP, &answer_ptr, NULL)) > 0))
     {
-    // This seems silly, but it defeats a strict-aliasing warning from gcc when
-    // a more conventional cast is used: (struct nihdns_header *)&answer
+    /*
+     * This seems silly, but it defeats a strict-aliasing warning from gcc when
+     * a more conventional cast is used: (struct nihdns_header *)&answer
+     */
     recast.char_ptr = answer;
     num_answers = ntohs((unsigned short)recast.header_ptr->ancount);
     num_cnames = 0;
@@ -1760,7 +1800,7 @@ int nihdns_a_inner(struct filter_settings *current_settings, char *target_name,
         }
 
     for (i = 0; (i < num_cnames) && !return_value; i++)
-      if ((size = nihdns_expand(answer, answer + answer_length, cname_ptr[i], (char *)host, MAX_HOSTNAME)) >= 0)
+      if ((size = nihdns_expand(current_settings, answer, answer + answer_length, cname_ptr[i], (char *)host, MAX_HOSTNAME, NULL)) >= 0)
         {
         current_lookup.data = target_name;
         current_lookup.count++;
@@ -1824,6 +1864,7 @@ int nihdns_mx(struct filter_settings *current_settings, char *target_name, struc
   int num_answers;
   int num_queries;
   int exit_loop;
+  int strlen_name;
   struct previous_action current_lookup;
   struct previous_action *tmp_lookup;
   char *target_name_array[2];
@@ -1846,8 +1887,10 @@ int nihdns_mx(struct filter_settings *current_settings, char *target_name, struc
   if ((target_name != NULL) &&
       ((answer_length = nihdns_query(current_settings, target_name_array, current_settings->current_options->nihdns_query_type_mx, NIHDNS_TYPE_MX, answer, MAX_DNS_PACKET_BYTES_UDP, &answer_ptr, NULL)) > 0))
     {
-    // This seems silly, but it defeats a strict-aliasing warning from gcc when
-    // a more conventional cast is used: (struct nihdns_header *)&answer
+    /*
+     * This seems silly, but it defeats a strict-aliasing warning from gcc when
+     * a more conventional cast is used: (struct nihdns_header *)&answer
+     */
     recast.char_ptr = answer;
     num_answers = ntohs((unsigned short)recast.header_ptr->ancount);
     num_cnames = 0;
@@ -1858,52 +1901,55 @@ int nihdns_mx(struct filter_settings *current_settings, char *target_name, struc
         {
         answer_ptr += size;
         type = NIHDNS_GETINT16(answer_ptr);
-        answer_ptr += sizeof(uint16_t);
+        answer_ptr += sizeof(uint16_t); /* type */
         answer_ptr += sizeof(uint16_t); /* class */
         answer_ptr += sizeof(uint32_t); /* ttl */
-        size = NIHDNS_GETINT16(answer_ptr);
-        answer_ptr += sizeof(uint16_t);
+        size = NIHDNS_GETINT16(answer_ptr); /* answer length */
+        answer_ptr += sizeof(uint16_t); /* answer length */
 
         switch (type)
           {
           case NIHDNS_TYPE_MX:
-            answer_ptr += sizeof(uint16_t);
+            answer_ptr += sizeof(uint16_t); /* MX preference */
 
-            if ((size = nihdns_expand(answer, answer + answer_length, answer_ptr, (char *)host, MAX_HOSTNAME)) > 0)
+            if ((size = nihdns_expand(current_settings, answer, answer + answer_length, answer_ptr, (char *)host, MAX_HOSTNAME, &strlen_name)) > 0)
               {
-              SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_MX, target_name, NIHDNS_GETINT16(answer_ptr - sizeof(uint16_t)), host);
-
-              if ((sscanf(host, "%3[0-9].%3[0-9].%3[0-9].%3[0-9]", ip_octets[0], ip_octets[1], ip_octets[2], ip_octets[3]) == 4) &&
-                  (sscanf(ip_octets[0], "%d", &ip_ints[0]) == 1) &&
-                  (ip_ints[0] >= 0) &&
-                  (ip_ints[0] <= 255) &&
-                  (sscanf(ip_octets[1], "%d", &ip_ints[1]) == 1) &&
-                  (ip_ints[1] >= 0) &&
-                  (ip_ints[1] <= 255) &&
-                  (sscanf(ip_octets[2], "%d", &ip_ints[2]) == 1) &&
-                  (ip_ints[2] >= 0) &&
-                  (ip_ints[2] <= 255) &&
-                  (sscanf(ip_octets[3], "%d", &ip_ints[3]) == 1) &&
-                  (ip_ints[3] >= 0) &&
-                  (ip_ints[3] <= 255))
+              if (strlen_name > 0)
                 {
-                SPAMDYKE_LOG_VERBOSE(current_settings, LOG_VERBOSE_MX_IP, host, target_name);
+                SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_MX, target_name, NIHDNS_GETINT16(answer_ptr - sizeof(uint16_t)), host);
 
-                if ((ip_ints[0] != localhost_octets[0]) ||
-                    (ip_ints[1] != localhost_octets[1]) ||
-                    (ip_ints[2] != localhost_octets[2]) ||
-                    (ip_ints[3] != localhost_octets[3]))
-                  return_value = 1;
+                if ((sscanf(host, "%3[0-9].%3[0-9].%3[0-9].%3[0-9]", ip_octets[0], ip_octets[1], ip_octets[2], ip_octets[3]) == 4) &&
+                    (sscanf(ip_octets[0], "%d", &ip_ints[0]) == 1) &&
+                    (ip_ints[0] >= 0) &&
+                    (ip_ints[0] <= 255) &&
+                    (sscanf(ip_octets[1], "%d", &ip_ints[1]) == 1) &&
+                    (ip_ints[1] >= 0) &&
+                    (ip_ints[1] <= 255) &&
+                    (sscanf(ip_octets[2], "%d", &ip_ints[2]) == 1) &&
+                    (ip_ints[2] >= 0) &&
+                    (ip_ints[2] <= 255) &&
+                    (sscanf(ip_octets[3], "%d", &ip_ints[3]) == 1) &&
+                    (ip_ints[3] >= 0) &&
+                    (ip_ints[3] <= 255))
+                  {
+                  SPAMDYKE_LOG_VERBOSE(current_settings, LOG_VERBOSE_MX_IP, host, target_name);
+
+                  if ((ip_ints[0] != localhost_octets[0]) ||
+                      (ip_ints[1] != localhost_octets[1]) ||
+                      (ip_ints[2] != localhost_octets[2]) ||
+                      (ip_ints[3] != localhost_octets[3]))
+                    return_value = 1;
+                  else
+                    SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_MX_LOCALHOST, host);
+                  }
                 else
-                  SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_MX_LOCALHOST, host);
+                  return_value = nihdns_a_inner(current_settings, (char *)host, NULL, NULL, 1, current_settings->current_options->nihdns_query_type_a);
                 }
               else
-                return_value = nihdns_a_inner(current_settings, (char *)host, NULL, NULL, 1, current_settings->current_options->nihdns_query_type_a);
+                SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_EMPTY_DATA, NULL);
 
               answer_ptr += size;
               }
-            else if (size == 0)
-              SPAMDYKE_LOG_EXCESSIVE(current_settings, LOG_DEBUGX_DNS_EMPTY_DATA, NULL);
             else if (size < 0)
               {
               SPAMDYKE_LOG_VERBOSE(current_settings, LOG_VERBOSE_DNS_COMPRESSION, target_name);
@@ -1949,7 +1995,7 @@ int nihdns_mx(struct filter_settings *current_settings, char *target_name, struc
         }
 
     for (i = 0; (i < num_cnames) && !return_value; i++)
-      if ((size = nihdns_expand(answer, answer + answer_length, cname_ptr[i], (char *)host, MAX_HOSTNAME)) >= 0)
+      if ((size = nihdns_expand(current_settings, answer, answer + answer_length, cname_ptr[i], (char *)host, MAX_HOSTNAME, NULL)) >= 0)
         {
         current_lookup.data = target_name;
         current_lookup.count++;
