@@ -1,6 +1,6 @@
 /*
   spamdyke -- a filter for stopping spam at connection time.
-  Copyright (C) 2014 Sam Clippinger (samc (at) silence (dot) org)
+  Copyright (C) 2015 Sam Clippinger (samc (at) silence (dot) org)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
@@ -153,7 +153,7 @@ void print_current_environment(struct filter_settings *current_settings)
       (current_settings->current_environment != NULL))
     for (i = 0; current_settings->current_environment[i] != NULL; i++)
       {
-      snprintf(tmp_buf, MAX_BUF, "%s\n%n", current_settings->current_environment[i], &tmp_strlen);
+      tmp_strlen = SNPRINTF(tmp_buf, MAX_BUF, "%s\n", current_settings->current_environment[i]);
       output_writeln(current_settings, LOG_ACTION_CURRENT_ENVIRONMENT, -1, tmp_buf, tmp_strlen);
       }
 
