@@ -1,6 +1,6 @@
 /*
   spamdyke -- a filter for stopping spam at connection time.
-  Copyright (C) 2014 Sam Clippinger (samc (at) silence (dot) org)
+  Copyright (C) 2015 Sam Clippinger (samc (at) silence (dot) org)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
@@ -113,7 +113,7 @@ int exec_checkpassword_argv(struct filter_settings *current_settings, char *file
             close(stdout_pipe[1]);
             close(stderr_pipe[1]);
 
-            snprintf(output_buf, MAX_CHECKPASSWORD, "%s%c%s%c%s%c%n", (username != NULL) ? username : "", '\0', (password != NULL) ? password : "", '\0', (timestamp != NULL) ? timestamp : "", '\0', &strlen_output_buf);
+            strlen_output_buf = SNPRINTF(output_buf, MAX_CHECKPASSWORD, "%s%c%s%c%s%c", (username != NULL) ? username : "", '\0', (password != NULL) ? password : "", '\0', (timestamp != NULL) ? timestamp : "", '\0');
             strlen_sent_buf = 0;
 
             start_time = time(NULL);
@@ -450,6 +450,7 @@ int exec_command_argv(struct filter_settings *current_settings, char *filename,
   int stdout_read_result;
   int stderr_read_result;
   pid_t child_pid;
+  pid_t wait_pid;
   char input_buf[MAX_COMMAND_BUF + 1];
   int strlen_input_buf;
   int status;
@@ -613,7 +614,11 @@ int exec_command_argv(struct filter_settings *current_settings, char *filename,
                     (protocol[current_protocol].type == ES_TYPE_SEND) &&
                     FD_ISSET(output_pipe[1], &write_fds))
                   {
-                  write(output_pipe[1], protocol[current_protocol].data, protocol[current_protocol].strlen_data);
+                  /*
+                   * This defeats an "unused result" warning from gcc.
+                   * Ignoring this result here is perfectly safe.
+                   */
+                  i = write(output_pipe[1], protocol[current_protocol].data, protocol[current_protocol].strlen_data);
                   current_protocol++;
                   }
 
@@ -655,7 +660,19 @@ int exec_command_argv(struct filter_settings *current_settings, char *filename,
               if (stderr_pipe[0] != -1)
                 close(stderr_pipe[0]);
 
-              if ((waitpid(child_pid, &status, WNOHANG) == child_pid) &&
+              do
+                {
+                if ((wait_pid = waitpid(child_pid, &status, WNOHANG)) == 0)
+                  {
+                  timeout.tv_sec = 0;
+                  timeout.tv_usec = TIMEOUT_COMMAND_EXIT_USECS;
+                  select(0, NULL, NULL, NULL, &timeout);
+                  }
+                }
+              while ((wait_pid == 0) &&
+                     ((time(NULL) - start_time) < TIMEOUT_COMMAND_SECS));
+
+              if ((wait_pid == child_pid) &&
                   WIFEXITED(status))
                 {
                 SPAMDYKE_LOG_DEBUG(current_settings, LOG_DEBUG_COMMAND_EXIT, WEXITSTATUS(status), new_filename);
