@@ -1,6 +1,6 @@
 /*
   spamdyke -- a filter for stopping spam at connection time.
-  Copyright (C) 2014 Sam Clippinger (samc (at) silence (dot) org)
+  Copyright (C) 2015 Sam Clippinger (samc (at) silence (dot) org)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
@@ -102,12 +102,12 @@ int find_path(struct filter_settings *current_settings, char *filename, char *en
         {
         if ((tmp_end = strchr(tmp_start, ENVIRONMENT_DELIMITER)) != NULL)
           {
-          snprintf(new_filename, MAX_BUF, "%.*s" DIR_DELIMITER_STR "%s%n", (int)(tmp_end - tmp_start), tmp_start, filename, &strlen_filename);
+          strlen_filename = SNPRINTF(new_filename, MAX_BUF, "%.*s" DIR_DELIMITER_STR "%s", (int)(tmp_end - tmp_start), tmp_start, filename);
           tmp_start = tmp_end + 1;
           }
         else
           {
-          snprintf(new_filename, MAX_BUF, "%s" DIR_DELIMITER_STR "%s%n", tmp_start, filename, &strlen_filename);
+          strlen_filename = SNPRINTF(new_filename, MAX_BUF, "%s" DIR_DELIMITER_STR "%s", tmp_start, filename);
           tmp_start = NULL;
           }
 
@@ -266,6 +266,7 @@ int examine_entry(char *target_string, int strlen_target_string, char *target_en
            (return_value == 0))
       if (((check_start &&
             ((start_wildcard_matches == NULL) ||
+             (tmp_string == target_string) ||
              (strchr(start_wildcard_matches, (tmp_string - 1)[0]) != NULL))) ||
            (!check_start &&
             (tmp_string == target_string))) &&
@@ -377,7 +378,7 @@ int sub_examine_tcprules_entry(char *destination, int size_destination, char *en
   char *loc_info;
   char *loc_start;
   char *tmp_name;
-  char ip_octets[4][7];
+  char ip_octets[4][8];
   char mask_octets[4][4];
   int ip_ints[4];
   int range_ints[4];
@@ -927,7 +928,7 @@ int read_file(struct filter_settings *current_settings, char *target_filename, c
 
   return_value = 0;
   line_num = 0;
-  usable_line_num = 0;
+  usable_line_num = start_index;
 
   if ((target_filename != NULL) &&
       (target_filename[0] != '\0') &&
@@ -959,15 +960,15 @@ int read_file(struct filter_settings *current_settings, char *target_filename, c
 
           if (end_line == -1)
             {
-            if ((tmp_array = (char **)realloc(*return_content, sizeof(char *) * ((line_num - zero_start) + 2))) != NULL)
+            if ((tmp_array = (char **)realloc(*return_content, sizeof(char *) * (usable_line_num + 2))) != NULL)
               {
-              tmp_array[line_num - zero_start] = NULL;
-              tmp_array[(line_num - zero_start) + 1] = NULL;
+              tmp_array[usable_line_num] = NULL;
+              tmp_array[usable_line_num + 1] = NULL;
               *return_content = tmp_array;
               }
             else
               {
-              SPAMDYKE_LOG_ERROR(current_settings, LOG_ERROR_MALLOC, sizeof(char *) * ((line_num - zero_start) + 2));
+              SPAMDYKE_LOG_ERROR(current_settings, LOG_ERROR_MALLOC, sizeof(char *) * (usable_line_num + 2));
               return_value = -1;
               break;
               }
@@ -975,9 +976,9 @@ int read_file(struct filter_settings *current_settings, char *target_filename, c
 
           if ((tmp_char = (char *)malloc(sizeof(char) * (strlen_line + 1))) != NULL)
             {
-            (*return_content)[line_num - zero_start] = tmp_char;
-            memcpy((*return_content)[line_num - zero_start], file_buf, sizeof(char) * strlen_line);
-            (*return_content)[line_num - zero_start][strlen_line] = '\0';
+            (*return_content)[usable_line_num] = tmp_char;
+            memcpy((*return_content)[usable_line_num], file_buf, sizeof(char) * strlen_line);
+            (*return_content)[usable_line_num][strlen_line] = '\0';
             }
           else
             {
@@ -1082,14 +1083,14 @@ char *reassemble_address(char *target_username, char *target_domain, char *missi
         (target_username[0] != '\0'))
       if ((target_domain != NULL) &&
           (target_domain[0] != '\0'))
-        snprintf(return_address, max_return_address, "%s@%s%n", target_username, target_domain, &tmp_strlen);
+        tmp_strlen = SNPRINTF(return_address, max_return_address, "%s@%s", target_username, target_domain);
       else
-        snprintf(return_address, max_return_address, "%s%n", target_username, &tmp_strlen);
+        tmp_strlen = SNPRINTF(return_address, max_return_address, "%s", target_username);
     else if ((target_domain != NULL) &&
              (target_domain[0] != '\0'))
-      snprintf(return_address, max_return_address, "@%s%n", target_domain, &tmp_strlen);
+      tmp_strlen = SNPRINTF(return_address, max_return_address, "@%s", target_domain);
     else if (missing_data != NULL)
-      snprintf(return_address, max_return_address, "%s%n", missing_data, &tmp_strlen);
+      tmp_strlen = SNPRINTF(return_address, max_return_address, "%s", missing_data);
     else
       return_address[0] = '\0';
     }
