@@ -1,6 +1,6 @@
 /*
   spamdyke -- a filter for stopping spam at connection time.
-  Copyright (C) 2014 Sam Clippinger (samc (at) silence (dot) org)
+  Copyright (C) 2015 Sam Clippinger (samc (at) silence (dot) org)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
@@ -63,7 +63,7 @@ int tls_password_callback(char *buf, int size, int rwflag, void *userdata)
   return_value = 0;
 
   if (((struct option_set *)userdata)->strlen_tls_privatekey_password > 0)
-    snprintf(buf, size, "%.*s%n", ((struct option_set *)userdata)->strlen_tls_privatekey_password, ((struct option_set *)userdata)->tls_privatekey_password, &return_value);
+    return_value = SNPRINTF(buf, size, "%.*s", ((struct option_set *)userdata)->strlen_tls_privatekey_password, ((struct option_set *)userdata)->tls_privatekey_password);
 
   return(return_value);
   }
@@ -617,14 +617,14 @@ ssize_t tls_read(struct filter_settings *current_settings, int target_fd, void *
             select(0, NULL, NULL, NULL, &tmp_timeval);
             break;
           default:
-            SPAMDYKE_LOG_VERBOSE(current_settings, LOG_ERROR_TLS_READ ": %s", tls_error(current_settings, return_value));
+            SPAMDYKE_LOG_VERBOSE(current_settings, LOG_VERBOSE_TLS_READ ": %s", tls_error(current_settings, return_value));
             continue_looping = 0;
             break;
           }
 
     if (continue_looping &&
         (return_value <= 0))
-      SPAMDYKE_LOG_VERBOSE(current_settings, LOG_ERROR_TLS_READ ": %s", tls_error(current_settings, return_value));
+      SPAMDYKE_LOG_VERBOSE(current_settings, LOG_VERBOSE_TLS_READ ": %s", tls_error(current_settings, return_value));
     }
   else
     return_value = read(target_fd, target_buf, num_bytes);
