@@ -1,6 +1,6 @@
 /*
   spamdyke -- a filter for stopping spam at connection time.
-  Copyright (C) 2014 Sam Clippinger (samc (at) silence (dot) org)
+  Copyright (C) 2015 Sam Clippinger (samc (at) silence (dot) org)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
@@ -98,7 +98,7 @@ void usage(struct filter_settings *current_settings, int text_level, char *error
              (current_settings->option_list[i].value_type == CONFIG_TYPE_NAME_MULTIPLE)) &&
             (current_settings->option_list[i].validity.string_list.strings[0] != NULL))
           {
-          snprintf(values_text, MAX_BUF, "%s%n", current_settings->option_list[i].validity.string_list.strings[0], &strlen_values_text);
+          strlen_values_text = SNPRINTF(values_text, MAX_BUF, "%s", current_settings->option_list[i].validity.string_list.strings[0]);
           for (j = 1; current_settings->option_list[i].validity.string_list.strings[j] != NULL; j++)
             {
             snprintf(values_text + strlen_values_text, MAX_BUF - strlen_values_text, USAGE_MESSAGE_NAME_VALUE_DELIMITER "%s", current_settings->option_list[i].validity.string_list.strings[j]);
